#: Available enums used by commands and server response status.

from .internals.commands.enums import (
    AlgorithmMethod,
    AlignLockStatus,
    ApplicationLayerBehavior,
    AutoNegFECOption,
    AutoNegFECStatus,
    AutoNegFECType,
    AutoNegMode,
    AutoNegStatus,
    AutoNegTecAbility,
    AutoOrManual,
    BRRMode,
    CaptureSize,
    ChassisSessionType,
    ChassisShutdownAction,
    CongestionType,
    CorruptionType,
    CustomDefaultCommand,
    CustomDefaultScope,
    EmbedIP,
    ErrorOnOff,
    FaultSignaling,
    FECMode,
    FilterMode,
    FilterType,
    FilterUse,
    FilterVlanType,
    HasDemo,
    HeaderLockStatus,
    IGMPVersion,
    ImpairmentLatencyMode,
    ImpairmentTypeIndex,
    InfiniteOrFinite,
    InfoAction,
    InjectErrorType,
    IsEnabled,
    IsOnline,
    IsPermanent,
    IsPresent,
    IsValid,
    LinkTrainFrameLock,
    L2PlusPresent,
    L3Present,
    L47IPVersion,
    L47PortSpeed,
    L47PortState,
    L47ProtocolType,
    L47TrafficState,
    LatencyMode,
    LatencyTypeCustomDist,
    LengthCheckType,
    LengthType,
    LicenseSpeed,
    LifecycleMode,
    LinkState,
    LinkTrainingFailureType,
    LinkTrainingInitCondition,
    LinkTrainingMode,
    LinkTrainingStatus,
    LinkTrainingStatusMode,
    LocalFaultStatus,
    LoopbackMode,
    LoopBandwidth,
    MDIXMode,
    MediaCFPState,
    MediaCFPType,
    MediaConfigurationType,
    ModifierAction,
    MSSType,
    MulticastExtOperation,
    MulticastHeaderFormat,
    MulticastOperation,
    NRZPreset,
    OnOff,
    OnOffDefault,
    OnOffWithSuppress,
    PacketDetailSelection,
    PacketType,
    PAM4FrameSize,
    PauseMode,
    PayloadGenerationMethod,
    PayloadMode,
    PayloadType,
    PFCMode,
    PHYSignalStatus,
    PolicerMode,
    PortSpeedMode,
    PPMSweepMode,
    PPMSweepStatus,
    PRBSInsertedType,
    PRBSInvertState,
    PRBSLockStatus,
    PRBSOnOff,
    PRBSPattern,
    PRBSPolynomial,
    PRBSStatisticsMode,
    ProtocolOption,
    ReconciliationSublayerSupport,
    RemoteFaultStatus,
    ReplayParserState,
    ReplaySchedulingMode,
    ReplaySyncBasedOn,
    ReservedAction,
    ReservedStatus,
    ResourceAllocationMode,
    RESTControlAction,
    Role,
    RTOType,
    RXCState,
    RXHState,
    SerdesStatus,
    ServiceStatus,
    ShadowWorkingSelection,
    SMAInputFunction,
    SMAOutputFunction,
    SMAStatus,
    SourceType,
    StartOrStop,
    StartTrigger,
    StopTrigger,
    SyncStatus,
    SystemUpdateStatus,
    TimeKeeperLicenseError,
    TimeKeeperLicenseFileState,
    TimeKeeperLicenseType,
    TimeKeeperServiceAction,
    TimeKeeperServiceStatus,
    TimeoutMode,
    Timescale,
    TimingSource,
    TLSVersion,
    TPLDMode,
    TrafficEngine,
    TrafficError,
    TrafficOnOff,
    TrafficScenario,
    TSNConfigProfile,
    TSNDeviationMode,
    TSNHistogramSource,
    TSNPortRole,
    TSNStatisticsTypes,
    TSNTimeSource,
    TXClockSource,
    TXClockStatus,
    TXCState,
    TXHState,
    TXMode,
    UpdateState,
    WhoClose,
    YesNo,
    Layer1ConfigType,
    LinkTrainAlgorithm,
    LinkTrainCmd,
    LinkTrainAnnounce,
    Layer1LogType,
    LinkTrainCmdFlags,
    LinkTrainCmdResults,
    LinkTrainCoeffs,
    LinkTrainEncoding,
    LinkTrainPresets,
    AnLtLogControl,
    RxEqExtCap,
    RxEqExtCapMode,
    PreCodingMode,
    GrayCodingMode,
    Endianness,
    StreamOption,
    DhcpState,
    DhcpVlanState,
    VlanType,
    ChassisModelNumber,
    ChassisModelName,
    ModuleModelName,
    FreyaAutonegMode,
    FreyaLinkTrainingMode,
    FreyaTecAbility,
    FreyaFECAbility,
    FreyaPauseAbility,
    FreyaTechAbilityHCDStatus,
    FreyaOutOfSyncPreset,
    Layer1Control,
    FreyaPCSVariant,
    FreyaTecAbilityHCD,
    Layer1Opcode,
)

__all__ = (
    "AlgorithmMethod",
    "AlignLockStatus",
    "ApplicationLayerBehavior",
    "AutoNegFECOption",
    "AutoNegFECStatus",
    "AutoNegFECType",
    "AutoNegMode",
    "AutoNegStatus",
    "AutoNegTecAbility",
    "AutoOrManual",
    "BRRMode",
    "CaptureSize",
    "ChassisSessionType",
    "ChassisShutdownAction",
    "CongestionType",
    "CorruptionType",
    "CustomDefaultCommand",
    "CustomDefaultScope",
    "EmbedIP",
    "ErrorOnOff",
    "FaultSignaling",
    "FECMode",
    "FilterMode",
    "FilterType",
    "FilterUse",
    "FilterVlanType",
    "HasDemo",
    "HeaderLockStatus",
    "IGMPVersion",
    "ImpairmentLatencyMode",
    "ImpairmentTypeIndex",
    "InfiniteOrFinite",
    "InfoAction",
    "InjectErrorType",
    "IsEnabled",
    "IsOnline",
    "IsPermanent",
    "IsPresent",
    "IsValid",
    "LinkTrainFrameLock",
    "L2PlusPresent",
    "L3Present",
    "L47IPVersion",
    "L47PortSpeed",
    "L47PortState",
    "L47ProtocolType",
    "L47TrafficState",
    "LatencyMode",
    "LatencyTypeCustomDist",
    "LengthCheckType",
    "LengthType",
    "LicenseSpeed",
    "LifecycleMode",
    "LinkState",
    "LinkTrainingFailureType",
    "LinkTrainingInitCondition",
    "LinkTrainingMode",
    "LinkTrainingStatus",
    "LinkTrainingStatusMode",
    "LocalFaultStatus",
    "LoopbackMode",
    "LoopBandwidth",
    "MDIXMode",
    "MediaCFPState",
    "MediaCFPType",
    "MediaConfigurationType",
    "ModifierAction",
    "MSSType",
    "MulticastExtOperation",
    "MulticastHeaderFormat",
    "MulticastOperation",
    "NRZPreset",
    "OnOff",
    "OnOffDefault",
    "OnOffWithSuppress",
    "PacketDetailSelection",
    "PacketType",
    "PAM4FrameSize",
    "PauseMode",
    "PayloadGenerationMethod",
    "PayloadMode",
    "PayloadType",
    "PFCMode",
    "PHYSignalStatus",
    "PolicerMode",
    "PortSpeedMode",
    "PPMSweepMode",
    "PPMSweepStatus",
    "PRBSInsertedType",
    "PRBSInvertState",
    "PRBSLockStatus",
    "PRBSOnOff",
    "PRBSPattern",
    "PRBSPolynomial",
    "PRBSStatisticsMode",
    "ProtocolOption",
    "ReconciliationSublayerSupport",
    "RemoteFaultStatus",
    "ReplayParserState",
    "ReplaySchedulingMode",
    "ReplaySyncBasedOn",
    "ReservedAction",
    "ReservedStatus",
    "ResourceAllocationMode",
    "RESTControlAction",
    "Role",
    "RTOType",
    "RXCState",
    "RXHState",
    "SerdesStatus",
    "ServiceStatus",
    "ShadowWorkingSelection",
    "SMAInputFunction",
    "SMAOutputFunction",
    "SMAStatus",
    "SourceType",
    "StartOrStop",
    "StartTrigger",
    "StopTrigger",
    "SyncStatus",
    "SystemUpdateStatus",
    "TimeKeeperLicenseError",
    "TimeKeeperLicenseFileState",
    "TimeKeeperLicenseType",
    "TimeKeeperServiceAction",
    "TimeKeeperServiceStatus",
    "TimeoutMode",
    "Timescale",
    "TimingSource",
    "TLSVersion",
    "TPLDMode",
    "TrafficEngine",
    "TrafficError",
    "TrafficOnOff",
    "TrafficScenario",
    "TSNConfigProfile",
    "TSNDeviationMode",
    "TSNHistogramSource",
    "TSNPortRole",
    "TSNStatisticsTypes",
    "TSNTimeSource",
    "TXClockSource",
    "TXClockStatus",
    "TXCState",
    "TXHState",
    "TXMode",
    "UpdateState",
    "WhoClose",
    "YesNo",
    "Layer1ConfigType",
    "LinkTrainAlgorithm",
    "LinkTrainCmd",
    "LinkTrainAnnounce",
    "Layer1LogType",
    "LinkTrainCmdFlags",
    "LinkTrainCmdResults",
    "LinkTrainCoeffs",
    "LinkTrainEncoding",
    "LinkTrainPresets",
    "AnLtLogControl",
    "RxEqExtCap",
    "RxEqExtCapMode",
    "PreCodingMode",
    "GrayCodingMode",
    "Endianness",
    "StreamOption",
    "DhcpState",
    "DhcpVlanState",
    "VlanType",
    "ChassisModelNumber",
    "ChassisModelName",
    "ModuleModelName",
    "FreyaAutonegMode",
    "FreyaLinkTrainingMode",
    "FreyaTecAbility",
    "FreyaFECAbility",
    "FreyaPauseAbility",
    "FreyaTechAbilityHCDStatus",
    "FreyaOutOfSyncPreset",
    "Layer1Control",
    "FreyaPCSVariant",
    "FreyaTecAbilityHCD",
    "Layer1Opcode",
)
